#include "args.h"

#include <argp.h>
#include <string.h>

static int parse_opt(int key, char *arg, struct argp_state *state) {
    struct Arguments *args = state->input;

    switch (key) {
        case 'i':
            args->behaviour = BEHAVIOUR_INTERPRET;
            return 0;

        case 'j':
            args->behaviour = BEHAVIOUR_JIT;
            args->jit.arch = parse_arch(arg);
            return 0;

        case 'a':
            args->behaviour = BEHAVIOUR_ASM;
            args->assemble.arch = parse_arch(arg);
            return 0;

        case ARGP_KEY_ARG:
            strncpy(args->program, arg, PATH_MAX);
            return 0;
    }

    return 0;
}

int parse_arguments(struct Arguments *args, int argc, char *argv[]) {
    struct argp_option options[] = {
        { "interpret", 'i', 0, 0, "Interpret the executable, instead of jitting it", 0 },
        { "jit", 'j', "ARCH", OPTION_ARG_OPTIONAL, "JIT the executable with instructions for ARCH [default: native]", 0 },
        { "asm", 'a', "ARCH", OPTION_ARG_OPTIONAL, "Assemble the executable with instructions for ARCH [default: native]", 0},
        { 0 }
    };

    const char *args_doc = "PROGRAM";

    struct argp argp = { options, parse_opt, args_doc, 0, 0, 0 };
    int result;
    argp_parse(&argp, argc, argv, 0, &result, args);
    return result;
}
